
--[[
	Little description to this 3 in 1 module.
	it register 4 modules. the first (name0) is only for configuration.
	all config variables in name0 are used in the other modules.
]]


----------------------------------
-- module independent variables --
----------------------------------
local addon, ns = ...
local C, L, I = ns.LC.color, ns.L, ns.I
local _

-----------------------------------------------------------
-- module own local variables and local cached functions --
-----------------------------------------------------------
local name0 = "GPS / Location / ZoneText" -- L["GPS / Location / ZoneText"]
local name1 = "GPS" -- L["GPS"]
local name2 = "Location" -- L["Location"]
local name3 = "ZoneText" -- L["ZoneText"]
local ldbName1, ldbName2, ldbName3 = name1, name2, name3
local ttName1, ttName2, ttName3 = name1.."TT", name2.."TT", name3.."TT"
local tt1, tt2, tt3, tooltip4, tooltip5
local tt5positions = {
	["LEFT"]   = {edgeSelf = "RIGHT",  edgeParent = "LEFT",   x = -2, y =  0},
	["RIGHT"]  = {edgeSelf = "LEFT",   edgeParent = "RIGHT",  x =  2, y =  0},
}

local iStr16 = "|T%s:16:16:0:0|t"
local iStr32 = "|T%s:32:32:0:0|t"
local gpsLoc = {
	zone = " ",
	color = "white",
	pvp = "contested",
	pos = ""
}
local zoneDisplayValues = {
	["1"] = L["Zone"],
	["2"] = L["Subzone"],
	["3"] = L["Zone"]..": "..L["Subzone"],
	["4"] = ("%s (%s)"):format(L["Zone"],L["Subzone"]),
	["5"] = ("%s (%s)"):format(L["Subzone"],L["Zone"]),
}
local teleports, portals, spells, items_for_menu,item_replacements = {},{},{},{},{}
for _,v in ipairs({18984,18986,21711,30542,30544,32757,35230,37863,40585,40586,43824,44934,44935,45688,45689,45690,45691,46874,48933,48954,48955,48956,48957,50287,51557,51558,51559,51560,52251,58487,63206,63207,63352,63353,63378,63379,64457,65274,65360,87215,95050,95051,95567,95568}) do items_for_menu[v] = true end
for _,v in ipairs({64488,28585,6948,44315,44314,37118}) do item_replacements[v] = true end


-- ------------------------------------- --
-- register icon names and default files --
-- ------------------------------------- --
I[name1] = {iconfile="Interface\\Addons\\"..addon.."\\media\\gps"}
I[name2] = {iconfile="Interface\\Addons\\"..addon.."\\media\\gps"}
I[name3] = {iconfile=GetItemIcon(11105),coords={0.05,0.95,0.05,0.95}}


---------------------------------------
-- module variables for registration --
---------------------------------------
ns.modules[name0] = {
	noBroker = true,
	desc = "",
	events = {
		--"PLAYER_LOGIN",
		--"PLAYER_ENTERING_WORLD",
		"ADDON_LOADED",
		--"ZONE_CHANGED",
		--"ZONE_CHANGED_INDOORS",
		--"ZONE_CHANGED_NEW_AREA",
		"LEARNED_SPELL_IN_TAB"
	},
	updateinterval = 0.1,
	config_defaults = {
		precision = 0,
		coordsFormat = "%s, %s",
		shortMenu = false
	},
	config_allowed = {
		coordsFormat = {
			["%s, %s"] = true,
			["%s / %s"] = true,
			["%s/%s"] = true,
			["%s | %s"] = true,
			["%s||%s"] = true
		}
	},
	config = {
		height = 62,
		elements = {
			{
				type	= "check",
				name	= "shortMenu",
				label	= L["Short transport menu"],
				desc	= L["Display the transport menu without names of spells and items behind the icons."],
			},
			{
				type	= "dropdown",
				name	= "coordsFormat",
				label	= L["Co-ordination format"],
				desc	= L["How would you like to view co-ordinations."],
				values	= {
					["%s, %s"]     = "10.3, 25.4",
					["%s / %s"]    = "10.3 / 25.4",
					["%s/%s"]      = "10.3/25.4",
					["%s | %s"]    = "10.3 | 25.4",
					["%s||%s"]     = "10.3||25.4"
				},
				default = "%s, %s",
			},
			{
				type		= "slider",
				name		= "precision",
				label		= L["Precision"],
				desc		= L["Change how much digits display after the dot."],
				minText		= "0",
				maxText		= "3",
				minValue	= 0,
				maxValue	= 3,
				default		= 0
			}
		}
	}
}

ns.modules[name1] = {
	desc = L["Broker to show the name of the current Zone and the co-ordinates."],
	events = {},
	updateinterval = nil,
	config_defaults = {
		bothZones = "2"
	},
	config_prepend = name0,
	config = {
		height = 68,
		elements = {
			{
				type = "dropdown",
				name = "bothZones",
				label = L["Display zone names"],
				desc = L["Display in broker zone and subzone if exists or one of it."],
				default = "2",
				values = zoneDisplayValues
			}
		}
	}
}

ns.modules[name2] = {
	desc = L["Broker to show your current co-ordinates within the zone."],
	enabled = false,
	events = {},
	updateinterval = nil,
	config_defaults = nil,
	config_prepend = name0,
	config = nil
}

ns.modules[name3] = {
	desc = L["Broker to show the name of the current zone."],
	enabled = false,
	events = {},
	updateinterval = nil,
	config_defaults = {
		bothZones = "2"
	},
	config_prepend = name0,
	config = {
		height = 68,
		elements = {
			{
				type = "dropdown",
				name = "bothZones",
				label = L["Display zone names"],
				desc = L["Display in broker zone and subzone if exists or one of it."],
				default = "2",
				values = zoneDisplayValues
			}
		}
	}
}


--------------------------
-- some local functions --
--------------------------
local function setSpell(tb,id)
	if IsSpellKnown(id) then
		local sName, _, icon, _, _, _, _, _, _ = GetSpellInfo(id)
		table.insert(tb,{id=id,icon32=iStr32:format(icon),icon16=iStr16:format(icon),name=sName,name2=sName})
	end
end
local function setItem(tb,id,nameReplacement)
	local itemName, _, _, _, _, _, _, _, _, icon, _ = GetItemInfo(id)
	table.insert(tb,{id=id,icon32=iStr32:format(icon),icon16=iStr16:format(icon),name=itemName,name2=(nameReplacement or itemName)})
end

local function chkInventory(ifExists)
	local found, count = {},0
	for bag = 0, NUM_BAG_SLOTS do
		for slot = 1, GetContainerNumSlots(bag) do
			local item_id = GetContainerItemID(bag,slot)
			if item_id then
				local itemName, _, _, _, _, _, _, _, _, icon, _ = GetItemInfo(item_id)
				if type(ifExists)=="table" and ifExists[item_id] then
					return itemName
				elseif type(ifExists)=="number" and ifExists == item_id then
					return itemName
				elseif items_for_menu[item_id]==true then
					setItem(found,item_id)
					count = count + 1
				elseif item_replacements[item_id]==true then
					setItem(found,item_id,GetBindLocation())
					count = count + 1
				end
			end
		end
	end
	return found, count
end

local function position()
	local p,f = Broker_EverythingDB[name0].precision, Broker_EverythingDB[name0].coordsFormat
	if not p then p = 0 end
	local precision_format = "%."..p.."f"
	if not f then f = "%s, %s" end

	local x, y = GetPlayerMapPosition("player")

	if x ~= 0 and y ~= 0 then
		return string.format(
			f,
			string.format(precision_format, (x * 100)),
			string.format(precision_format, (y * 100))
		)
	else
		local pX = strrep("?",p)
		return string.format(f, (pX~="" and "?."..pX or "?"), (pX~="" and "?."..pX or "?") )
	end
end

local function zone(byName)
	local subZone = GetSubZoneText() or ""
	local zone = GetRealZoneText() or ""
	local types = {"%s: %s","%s (%s)"}

	if Broker_EverythingDB[byName].bothZones=="2" and subZone~="" then
		return subZone
	elseif Broker_EverythingDB[byName].bothZones=="3" and subZone~="" then
		return subZone and types[1]:format(zone,subZone or "")
	elseif Broker_EverythingDB[byName].bothZones=="4" and subZone~="" then
		return subZone and types[2]:format(zone,subZone)
	elseif Broker_EverythingDB[byName].bothZones=="5" and subZone~="" then
		return subZone and types[2]:format(subZone,zone)
	end

	return zone
end

local function zoneColor()
	local p, _, f = GetZonePVPInfo()
	local color = "white"

	if p == "combat" or p == "arena" or p == "hostile" then
		color = "red"
	elseif p == "contested" or p == nil then 
		color = "dkyellow"
		p = "contested"
	elseif p == "friendly" then
		color = "ltgreen"
	elseif p == "sanctuary" then
		color = "ltblue"
	end
	--[[
		L["Contested"]
		L["Sanctuary"]
		L["Friendly"]
		L["Combat"]
		L["Arena"]
		L["Hostile"]
	]]
	return p, color
end

-- shared tooltip for modules Location, GPS and ZoneText
local function gpsTooltip(self,tt,modName)
	if (not tt.key) or tt.key~=modName.."TT" then return end -- don't override other LibQTip tooltips...

	local pvp, color = zoneColor()
	local line, column
	pvp = gsub(pvp,"^%l", string.upper)

	if buttonFrame then buttonFrame:ClearAllPoints() buttonFrame:Hide() end

	tt:Clear()

	tt:AddHeader(C("dkyellow",L[modName]))

	tt:AddSeparator()

	local lst = {
		{C("ltyellow",L["Zone"] .. ":"),GetRealZoneText()},
		{C("ltyellow",L["Subzone"] .. ":"),GetSubZoneText()},
		{C("ltyellow",L["Zone status"] .. ":"),C(color,L[pvp])},
		{C("ltyellow",L["Co-ordinates"] .. ":"),position() or C(gpsLoc.posColor or gpsLoc.color,gpsLoc.pos)}
	}

	for _, d in pairs(lst) do
		line, column = tt:AddLine()
		tt:SetCell(line,1,d[1],nil,nil,2)
		tt:SetCell(line,3,d[2],nil,nil,1)
	end

	if gpsLoc.posColor then
		line,column = tt:AddLine()
		tt:SetCell(line,1,C(gpsLoc.posColor,gpsLoc.posInfo),nil,"CENTER",3)
	end

	tt:AddSeparator()

	line, column = tt:AddLine()
	tt:SetCell(line,1,C("ltyellow",L["Inn"]..":"),nil,nil,1)
	tt:SetCell(line,2,GetBindLocation(),nil,nil,2)

	local item = chkInventory(item_replacements)
	if type(item)=="string" then
		tt:SetLineScript(line,"OnEnter",function(self)
			--buttonHandler(self,"item",item)
			ns.secureButton(self,{ {typeName="type", typeValue="item", attrName="item", attrValue=item} }, modName.."_Inn")
		end)
	end

	if Broker_EverythingDB.showHints then
		tt:AddSeparator(3,0,0,0,0)
		line, column = tt:AddLine()
		tt:SetCell(line, 1, C("copper",L["Left-click"]).." || "..C("green",L["Open World map"]), nil, nil, 3)
		line, column = tt:AddLine()
		tt:SetCell(line, 1, C("copper",L["Right-click"]).." || "..C("green",L["Open transport menu"]), nil, nil, 3)
	end
end

local function gpsTooltip2(tt)
	local pts,ipts,tls,itls = {},{},{},{}
	local line, column,cellcount = nil,nil,5
	local inv, inv_c = chkInventory()

	local function add_title(title)
		tt:AddSeparator(4,0,0,0,0)
		tt:AddLine(C("ltyellow",title))
		tt:AddSeparator()
	end

	local function add_cell(v,t)
		tt:SetCell(line, cellcount, v.icon32, nil, nil, 1)
		tt:SetCellScript(line,cellcount,"OnEnter",function(self) ns.secureButton(self,{ {typeName="type", typeValue=t, attrName=t, attrValue=v.name} }) end)
	end

	local function add_line(v,t)
		line, column = tt:AddLine(v.icon16..(v.name2 or v.name))
		tt:SetLineScript(line,"OnEnter",function(self) ns.secureButton(self,{ {typeName="type", typeValue=t, attrName=t, attrValue=v.name} }) end)
	end

	local function add_obj(v,t)
		if Broker_EverythingDB[name0].shortMenu then
			if cellcount<4 then
				cellcount = cellcount + 1
			else
				cellcount = 1
				line, column = tt:AddLine()
			end
			add_cell(v,t)
		else
			add_line(v,t)
		end
	end

	tt:Clear()

	-- title
	if not Broker_EverythingDB[name0].shortMenu then
		tt:AddHeader(C("dkyellow","Choose your transport"))
	end

	local counter = 0

	if #teleports>0 or #portals>0 or #spells>0 then
		-- class title
		if not Broker_EverythingDB[name0].shortMenu then
			add_title(ns.player.classLocale)
		end
		-- class spells
		if ns.player.class=="MAGE" then
			for i,v in ns.pairsByKeys(teleports) do
				add_obj(v,"spell")
			end
			if not Broker_EverythingDB[name0].shortMenu then
				tt:AddSeparator()
			end
			for i,v in ns.pairsByKeys(portals) do
				add_obj(v,"spell")
			end
		else
			for i,v in ns.pairsByKeys(spells) do
				add_obj(v,"spell")
			end
		end
	end

	if inv_c>0 then
		-- item title
		if not Broker_EverythingDB[name0].shortMenu then
			add_title(L["Items"])
		end
		-- items
		for i,v in ns.pairsByKeys(inv) do
			add_obj(v,"item")
			counter = counter + 1
		end
	end
	
	if counter==0 then
		tt:AddSeparator(4,0,0,0,0)
		tt:AddHeader(C("ltred",L["Sorry"].."!"))
		tt:AddSeparator(1,1,.4,.4,1)
		tt:AddLine(C("ltred",L["No spells or items found"].."."))
	end
end

local function onleave(self,tt,ttN)
	if tt then
		if MouseIsOver(tt) then
			tt:SetScript('OnLeave', function(self)
				ns.hideTooltip(self,ttN)
				ns.secureButton(false)
			end)
		else
			ns.hideTooltip(tt,ttN)
			ns.secureButton(false)
		end
	end
	if tooltip4 then
		if MouseIsOver(tooltip4) then
			ns.hideTooltip(tooltip4,name1)
			tt:SetScript('OnLeave',function(self)
				ns.hideTooltip(tooltip4,name1)
			end)
		else
			ns.hideTooltip(tooltip4,name1)
			ns.secureButton(false)
		end
	end
	ns.secureButton(false)
end

local function onclick(self,button)
	if button == "LeftButton" then
		if GUILockDown == nil then 
			securecall("ToggleFrame",WorldMapFrame)
		elseif GUILockDown == 1 then 
			return 
		end
	elseif button == "RightButton" then
		if tt1 then
			onleave(self,tt1)
		end
		if tt2 then
			onleave(self,tt2)
		end
		if tt3 then
			onleave(self,tt3)
		end
		if Broker_EverythingDB[name0].shortMenu then
			tooltip4 = ns.LQT:Acquire(name1.."TT2", 4, "LEFT","LEFT","LEFT","LEFT")
		else
			tooltip4 = ns.LQT:Acquire(name1.."TT2", 1, "LEFT")
		end
		gpsTooltip2(tooltip4)
		ns.createTooltip(self,tooltip4)
	end
end


------------------------------------
-- module (BE internal) functions --
------------------------------------

ns.modules[name0].init = function(self)
	ldbName1 = (Broker_EverythingDB.usePrefix and "BE.." or "")..name1
	ldbName2 = (Broker_EverythingDB.usePrefix and "BE.." or "")..name2
	ldbName3 = (Broker_EverythingDB.usePrefix and "BE.." or "")..name3
end

--[[ ns.modules[name1].init = function(self) end ]]

--[[ ns.modules[name2].init = function(self) end ]]

--[[ ns.modules[name3].init = function(self) end ]]

ns.modules[name0].onevent = function(self,event,msg)
	teleports, portals, spells = {},{},{}
	if ns.player.class=="MAGE" then
		for _,v in ipairs({3561,3562,3563,3565,3566,3567,32271,32272,33690,35715,49358,49359,53140,88342,88344,120145,132621,132627}) do setSpell(teleports,v) end
		for _,v in ipairs({10059,11416,11417,11418,11419,11420,32266,32267,33691,35717,49360,49361,53142,88345,88346,120146,132620,132626}) do setSpell(portals,v) end
	end
	for _,v in ipairs({50977,18960,556,126892}) do setSpell(spells,v) end
end

--[[ ns.modules[name1].onevent = function(self,event,msg) end ]]

--[[ ns.modules[name2].onevent = function(self,event,msg) end ]]

--[[ ns.modules[name3].onevent = function(self,event,msg) end ]]

--[[ ns.modules[name0].onmousewheel = function(self,direction) end ]]

--[[ ns.modules[name1].onmousewheel = function(self,direction) end ]]

--[[ ns.modules[name2].onmousewheel = function(self,direction) end ]]

--[[ ns.modules[name3].onmousewheel = function(self,direction) end ]]

ns.modules[name0].onupdate = function(self)
	gpsLoc.zone1 = zone(name1)
	gpsLoc.zone3 = zone(name3)
	gpsLoc.pvp, gpsLoc.color = zoneColor()
	local pos = position()
	if pos then
		gpsLoc.pos = pos
		gpsLoc.posColor = nil
		gpsLoc.posInfo = nil
	else
		if gpsLoc.posLast==nil then
			gpsLoc.posLast=time()
		elseif time()-gpsLoc.posLast>5 then
			gpsLoc.posColor = "orange"
			gpsLoc.posInfo = L["Co-ordinates indeterminable"]
		end
	end
	local obj,data = nil,{
		[name1] = C(gpsLoc.color,gpsLoc.zone1.." (")..C(gpsLoc.posColor or gpsLoc.color,gpsLoc.pos)..C(gpsLoc.color,")"),
		[name2] = C(gpsLoc.posColor or gpsLoc.color,gpsLoc.pos),
		[name3] = C(gpsLoc.color,gpsLoc.zone3)
	}
	for n,d in pairs(data) do
		obj = ns.LDB:GetDataObjectByName(n) or { text = "" }
		obj.text = d
	end
end

--[[ ns.modules[name1].onupdate = function(self) end ]]

--[[ ns.modules[name2].onupdate = function(self) end ]]

--[[ ns.modules[name3].onupdate = function(self) end ]]

--[[ ns.modules[name0].optionspanel = function(panel) end ]]

--[[ ns.modules[name1].optionspanel = function(panel) end ]]

--[[ ns.modules[name2].optionspanel = function(panel) end ]]

--[[ ns.modules[name3].optionspanel = function(panel) end ]]


-------------------------------------------
-- module functions for LDB registration --
-------------------------------------------
ns.modules[name1].onenter = function(self)
	tt1 = ns.LQT:Acquire(ttName1, 3, "LEFT", "RIGHT", "RIGHT")
	gpsTooltip(self,tt1,name1)
	ns.createTooltip(self,tt1)
end

ns.modules[name2].onenter = function(self)
	tt2 = ns.LQT:Acquire(ttName2, 3, "LEFT", "RIGHT", "RIGHT")
	gpsTooltip(self,tt2,name2)
	ns.createTooltip(self,tt2)
end

ns.modules[name3].onenter = function(self)
	tt3 = ns.LQT:Acquire(ttName3, 3, "LEFT", "RIGHT", "RIGHT")
	gpsTooltip(self,tt3,name3)
	ns.createTooltip(self,tt3)
end

ns.modules[name1].onleave = function(self)
	onleave(self,tt1,name1)
end

ns.modules[name2].onleave = function(self)
	onleave(self,tt2,name2)
end

ns.modules[name3].onleave = function(self)
	onleave(self,tt3,name3)
end

ns.modules[name1].onclick = onclick

ns.modules[name2].onclick = onclick

ns.modules[name3].onclick = onclick

--[[ ns.modules[name1].ondblclick = function(self,button) end ]]

--[[ ns.modules[name2].ondblclick = function(self,button) end ]]

--[[ ns.modules[name3].ondblclick = function(self,button) end ]]
