--[[
]]

if true then return end

local MAJOR, MINOR = "LibOptionPanel-2.0", 1

if not ( LibStub ) then error( MAJOR .. "requires LibStub") end

local lib, oldLib = LibStub:NewLibrary(MAJOR, MINOR)

if not ( lib ) then return end

local function IsBuildOrHigher(...) -- check if version or higher
	local args, result, ver, build = {...}, true, GetBuildInfo()

	-- create a table like { 5, 4, 8, 12345 } for walkthrough
	ver = strsplit(".",ver);
	tinsert(ver,build);

	for i, v in ipairs(args)do
		-- run never longer then necessary. 5 elements in ver and stop on first nil argument.
		if not ( v ) or i>#ver then
			break;
		end
		if tonumber(v)<tonumber(ver[i]) then
			result = false;
		end
	end

	return result;
end

lib.panels = {}

local meta = setmetatable({},{__call=function(t,...) return t:CreatePanel(...) end})

lib.panelMeta = meta

function meta:chkArgs(func, args, definitions)
	for i,v in pairs(definitions) do
		if ( args[i] ) then
			if ( v.onFail=="error" ) then
				if ( v.type ) then
					assert(type(args[i])==v.type, "argument '"..i.."' wrong type (expected "..v.type..", given "..type(args[i])..")");
				end
				if ( v.match ) and ( args[i]:match(v.match) ) then
					error("argument '"..i.."' must be matching with '"..v.match.."'!",2)
				end
			end
		elseif ( v.need ) then
			error("missing argument "..i.." in "..MAJOR..".<panel>:"..func.."()",2);
		end
	end
end

function meta:Refresh()
end

function meta:CreatePanel(args)
end

function meta:CreateSubPanel(args)
end

--[[
	function <panel>:CreateText()
	@param table
		table_key text     <string> {must have}
		table_key font     <string> {optional}   ; like GameFontNormalLarge          ; default: GameFontNormal    ; silent fallback to default
		table_key justifyH <string> {optional}   ; only LEFT, CENTER or RIGHT        ; default: LEFT              ; silent fallback to default
		table_key justityV <string> {optional}   ; only TOP, MIDDLE or BOTTOM        ; default: TOP               ; silent fallback to default
		table_key point    <table>  {optional}   ; see return of <frame>:GetPoint()  ; default: TOPLEFT, 16, -16  ; silent fallback to default
	@return 
]]
function meta:CreateText(args)
	self:chkArgs("CreateText",args,{
		text     = { type = "string", need = true },
		justifyH = { type = "string", match = "^LEFT||RIGHT||CENTER$", default="LEFT",             onFail="default" },
		justifyV = { type = "string", match = "^TOP||MIDDLE||BOTTOM$", default="TOP",              onFail="default" },
		point    = { type = "table",                                   default={"TOPLEFT",16,-16}, onFail="default" }
	});

	local obj = self:CreateFontString(nil, "ARTWORK", type(args.font)=="string" or "GameFontNormal")
	obj:SetText(tostring(args.text));
	obj:SetJustifyH(args.justifyH);
	obj:SetJustifyV(args.justifyV);
	obj:SetPoint(unpack(args.point));
	return obj;
end

function meta:CreateScrollFrame(args)
end

function meta:CreateSlider(args)
end

function meta:CreateDropDown(args)
end

function meta:CreateToggle(args)
end

function meta:CreateColorPicker(args)
end

function meta:CreateEditBox(args)
end

